extern "C" {

#include "disp_tools.hpp"
#include "display_syscalls.h"
#include "keyboard_syscalls.h"
#include "STD_syscalls.h"
#include "misc_tools.h"

#include "fargoal.hpp"

void i_SetPointVRAM( int x1, int y1, int mode )
{
	Bdisp_SetPointWB_VRAM( x1, y1, mode );
};

void Bdisp_DrawLineVRAM( int x1, int y1, int x2, int y2 ){
TShape shape;
unsigned char work[28];
	shape.x1 = x1;
	shape.x2 = x2;
	shape.y1 = y1;
	shape.y2 = y2;
	shape.f[0] = 2;
	shape.f[1] = 2;
	shape.f[2] = 1;
	shape.f[3] = 1;
	work[0] = 1;
	Bdisp_ShapeBaseWB( work, &shape, 0, 1, 0, 0 );
};

//
char*memput( void*t, void*s, int n ){
	memcpy( t, s, n );
	return (char*)t + n;
}

//
char*memget( void*t, void*s, int n ){
	memcpy( t, s, n );
	return (char*)s + n;
}

//                                                                       T W o r l d I t e m 
TWorldItem::TWorldItem(){
	SetPos( 0 , 0 );
	SetWidth( 1, 1 );
	SetType( witUNKNOWN );
	ContentType = ttUNKNOWN;
	status = 0;
	flickering = 0;
	flicker_timer = 0;
}

//
void TWorldItem::SetType( int type ){
	FType = type;
}	

//
int TWorldItem::GetType(){
	return FType;
}

//
void TWorldItem::SetPos( int x, int y ){
	Px = x << MINI_TO_MAXI_PIX_BITS;
	Py = y << MINI_TO_MAXI_PIX_BITS;
}

//
void TWorldItem::SetWidth( int dx, int dy ){
	Pxw = dx;
	Pyw = dy;
}

//
int TWorldItem::GetX( int offset ){
	return ( Px + offset + ( 1 << MINI_TO_MAXI_PIX_BITS ) -1  ) >> MINI_TO_MAXI_PIX_BITS;
}

//
int TWorldItem::GetY( int offset ){
	return ( Py + offset + ( 1 << MINI_TO_MAXI_PIX_BITS ) -1 ) >> MINI_TO_MAXI_PIX_BITS;
}

//
void TWorldItem::SetX( int value ){
	Px = value << MINI_TO_MAXI_PIX_BITS;
}

//
void TWorldItem::SetY( int value ){
	Py = value << MINI_TO_MAXI_PIX_BITS;
}

//
int TWorldItem::GetWidthX(){
	return Pxw;
}

//
int TWorldItem::GetWidthY(){
	return Pyw;
}

//
int TWorldItem::GetX2( int offset ){
	return GetX( offset ) + GetWidthX();
}

//
int TWorldItem::GetY2( int offset ){
	return GetY( offset ) + GetWidthY();
}

//
void TWorldItem::Draw(  int mode  ){
int x, y;
	x = GetX();
	y = GetY();
    Bdisp_SetPoint_VRAM( x - Pxw, y - Pyw, mode );
    Bdisp_SetPoint_VRAM( x - Pxw, y + Pyw, mode );
    Bdisp_SetPoint_VRAM( x + Pxw, y - Pyw, mode );
    Bdisp_SetPoint_VRAM( x + Pxw, y + Pyw, mode );
}

//
int TWorldItem::Known( int mode ){
	switch( mode ){
		case 0 : status &= ~STATUS_KNOWN;
			break;
		case 1 : status |= STATUS_KNOWN;
			break;
	}
	return status & STATUS_KNOWN;
};


#define xDEBUG

//
int TWorldItem::InsideOfBounds(){
	if ( GetX() > ( Pxw - 1 ) ) 
		if ( GetX() < ( RIGHTBORDER - Pxw ) ) 
			if ( GetY() > ( Pyw - 1 ) ) 
				if ( GetY() < ( BOTTOMBORDER - Pyw ) ) return 1;
	return 0;
}

//
int TWorldItem::SizeOfSaveBlock(){
int result = 0;
	result += sizeof( status );
	result += sizeof( ContentType );
	result += sizeof( ContentValue );
	result += sizeof( Px );
	result += sizeof( Py );
	result += sizeof( Pxw );
	result += sizeof( Pyw );
	result += sizeof( FType );

	return result;
};

//
int TWorldItem::SaveData( char*p ){
	MEMPUT(status);
	MEMPUT(ContentType);
	MEMPUT(ContentValue);
	MEMPUT(Px);
	MEMPUT(Py);
	MEMPUT(Pxw);
	MEMPUT(Pyw);
	MEMPUT(FType);
	
	return (int)p;
};

//
int TWorldItem::LoadData( char*p ){
	MEMGET(status);
	MEMGET(ContentType);
	MEMGET(ContentValue);
	MEMGET(Px);
	MEMGET(Py);
	MEMGET(Pxw);
	MEMGET(Pyw);
	MEMGET(FType);

	return (int)p;
};

//                                                                       T B o u l d e r
TBoulder::TBoulder(){
	SetType( witBOULDER );
}

//
void TBoulder::Draw(  int mode  ){
int x1, x2, y1, y2;
	x1 = GetX();
	x2 = GetX2();
	y1 = GetY();
	y2 = GetY2();
	Bdisp_DrawLineVRAM( x1+1, y1, x2-1, y1 );
	y1++;
	while ( y1 < y2 ){
		Bdisp_DrawLineVRAM( x1, y1, x2, y1 );
		y1++;
	}
	Bdisp_DrawLineVRAM( x1+1, y1, x2-1, y1 );
	
	if ( mode & 2 ){
		y1 = GetY();
		while ( y1 <= y2 ){
			x1 = GetX();
			while ( x1 <= x2 ){
				if ( x1 == GetX() ){ if ( random() < 0x1000 ) i_SetPointVRAM( x1, y1, random() ); }
				else if ( x1 == x2 ){ if ( random() < 0x1000 ) i_SetPointVRAM( x1, y1, random() ); }
				else if ( y1 == GetY() ){ if ( random() < 0x1000 ) i_SetPointVRAM( x1, y1, random() ); }
				else if ( y1 == y2 ){ if ( random() < 0x1000 ) i_SetPointVRAM( x1, y1, random() ); }
				else switch ( mode & ~0x03 ){
					case 0x0100 : break;
					case 0x0200 : if ( random() < 0x2000 ) i_SetPointVRAM( x1, y1, random() ); break;
					case 0x0300 : if ( random() < 0x3000 ) i_SetPointVRAM( x1, y1, random() ); break;
					case 0x0400 : if ( random() < 0x4000 ) i_SetPointVRAM( x1, y1, random() ); break;
					case 0x0500 : if ( random() < 0x5000 ) i_SetPointVRAM( x1, y1, random() ); break;
				}
				x1++;
			}
			y1++;
		}
	}
}						

//
int TBoulder::SizeOfSaveBlock(){
int result = TWorldItem::SizeOfSaveBlock();
	return result;
};

//
int TBoulder::SaveData( char*p ){
	p = (char*)TWorldItem::SaveData( p );
	return (int)p;
};

//
int TBoulder::LoadData( char*p ){
	p = (char*)TWorldItem::LoadData( p );
	return (int)p;
};

//
void G3A_main( void ){
TWorldItem wi;
TBoulder bd;
int key;
	Bdisp_AllClr_VRAM();

	wi.SetPos( 200 , 200 );
	wi.SetWidth( 10, 10 );
	wi.Draw( 0x001F );
	
	bd.SetPos( 100 , 100 );
	bd.SetWidth( 20, 20 );
	bd.Draw( 2 );
	
	GetKey( &key );

}

#pragma section _BR_Size
unsigned long BR_Size;
#pragma section

}

