extern "C" {

#include "keyboard.hpp"
#include "keyboard_syscalls.h"
#include "disp_tools.hpp"
#include "display_tools.h"
#include "app_syscalls.h"
#include "convert_syscalls.h"
#include "STD_syscalls.h"
#include "filebios.h"
#include "BFILE_syscalls.h"
#include "RTC_syscalls.h"
#include "convert_tools.h"
#include "EditStr.hpp"
#include "Serial_syscalls.h"

#define KEY_HELP KEY_CTRL_F1
#define KEY_SEARCH KEY_CTRL_F2
#define KEY_BACKUP KEY_CTRL_F3
#define KEY_SERIAL KEY_CTRL_F4
#define KEY_HEXTOGGLE KEY_CTRL_F6
#define KEY_LEFT KEY_CTRL_LEFT
#define KEY_RIGHT KEY_CTRL_RIGHT
#define KEY_UP KEY_CTRL_UP
#define KEY_DOWN KEY_CTRL_DOWN
#define KEY_0 KEY_CHAR_0
#define KEY_1 KEY_CHAR_1
#define KEY_2 KEY_CHAR_2
#define KEY_3 KEY_CHAR_3
#define KEY_4 KEY_CHAR_4
#define KEY_5 KEY_CHAR_5
#define KEY_6 KEY_CHAR_6
#define KEY_7 KEY_CHAR_7
#define KEY_8 KEY_CHAR_8
#define KEY_9 KEY_CHAR_9
#define KEY_A KEY_CTRL_XTT
#define KEY_B KEY_CHAR_LOG
#define KEY_C KEY_CHAR_LN
#define KEY_D KEY_CHAR_SIN
#define KEY_E KEY_CHAR_COS
#define KEY_F KEY_CHAR_TAN
#define KEY_EXIT KEY_CTRL_EXIT
#define KEY_EXE KEY_CTRL_EXE
#define KEY_DEL KEY_CTRL_DEL
#define KEY_RECURRENT KEY_CTRL_OPTN

#define BLOCKSIZE 0x1000
#define DIVISOR 0x1400

const unsigned char FNBackup[] = "\\\\fls0\\MEMORY.BKP";
//
void BackupMemory( const unsigned int start_address ){
int size=0x10000, blocksize;
int i, handle;
unsigned short FName[64];
unsigned char hb[20];
void*work_address=(void*)start_address;
TEditStr ed;
	memset( hb, 0, 20 );
	memset( hb, '0', 8 );

	ed.xpos = 3;
	ed.ypos = 2;
	ed.height = 4;
	ed.buffer = hb;
	if ( !EditStr_Execute( "enter size:", &ed ) ) return;
	
	size = HexToInt( hb );
	size = ( size + 1 ) & ~1;
	if ( size < 0 ) return;
	// not more than 2 MB
	if ( size > 0x200000 ) return;
	
	MsgBoxPush( 3 );
	
	locate_OS( 3, 3 );
	PrintLine( "Creating...wait", 19 );
	locate_OS( 3, 4 );
	itoa( 2 + size / DIVISOR, hb );
	Print( hb );
	PrintLine( " sec", 19 );
	Bdisp_PutDisp_DD();
	
	Bfile_StrToName_ncpy( FName, FNBackup, 64 );
	Bfile_DeleteEntry( FName );
	if (Bfile_CreateEntry_OS( FName, 1, &size )){
		locate_OS( 3, 3 );
		PrintLine( "Create failed!", 19 );
		GetKey( &i );
	}else{
		locate_OS( 3, 3 );
		PrintLine( "Writing...", 19 );
		handle = Bfile_OpenFile_OS( FName, 2 );
		if (handle >= 0){
			while (size){
				itoa( size, hb );
				locate_OS( 3, 4 );
				PrintLine( hb, 19 );
				Bdisp_PutDisp_DD();

				if (size >= BLOCKSIZE) blocksize = BLOCKSIZE;
				else blocksize = size;
				Bfile_WriteFile_OS( handle, (const void *)work_address, blocksize );
				work_address = (void*)((unsigned int)work_address + BLOCKSIZE);
				size = size - blocksize;
			}
			Bfile_CloseFile_OS( handle );
		}else{
			locate_OS( 3, 3 );
			PrintLine( "Open failed!", 19 );
			GetKey( &i );
		}
	}
	MsgBoxPop();
	Cursor_SetFlashOn( 0 );

}

//
void MemoryToSerial( const unsigned int start_address ){
int j, i, size, iresult, key;
unsigned int chksum;
unsigned char hb[20];
void*work_address=(void*)start_address;
TEditStr ed;
unsigned char serial_mode[ 6 ];
unsigned char*p;

	memset( hb, 0, 20 );
	memset( hb, '0', 8 );

	ed.xpos = 3;
	ed.ypos = 2;
	ed.height = 4;
	ed.buffer = hb;
	if ( !EditStr_Execute( "enter size:", &ed ) ) return;
	
	size = HexToInt( hb );
	if ( size < 0 ) return;
	// not more than 32 MB
	if ( size > 0x2000000 ) return;
	
	MsgBoxPush( 4 );
	
	locate_OS( 3, 3 );
	PrintLine( "Opening...wait", 19 );
	locate_OS( 3, 4 );
	itoa( 2 + size / DIVISOR, hb );
	Print( hb );
	PrintLine( " sec", 19 );
	Bdisp_PutDisp_DD();

	// open serial	
	serial_mode[0] = 0;
	serial_mode[1] = 9;
	serial_mode[2] = 2;
	serial_mode[3] = 0;
	serial_mode[4] = 1;
	serial_mode[5] = 0;
	iresult = Serial_Open( serial_mode );

	if ( iresult ){
		locate_OS( 3, 3 );
		PrintLine( "Serial failed!", 19 );
		GetKey( &i );
	}else{
		locate_OS( 3, 3 );
		PrintLine( "Sending...", 19 );
		locate_OS( 3, 4 );
		IntToHex( size, hb );
		hb[ 8 ] = 0;
		Print( hb );
		locate_OS( 3, 5 );
		IntToHex( (unsigned int)work_address, hb );
		hb[ 8 ] = 0;
		Print( hb );
//		key = 0;
//		while ( key != KEY_CTRL_F1 ) GetKey( &key );
		p = (unsigned char*)&size;
		for (j=0;j<2;j++){
			for (i=0;i<4;i++){
				iresult = 1;
				while( iresult == 1 ) iresult = Serial_DirectTransmitOneByte( p[i] );
			}
			if ( iresult ) return;
		}
		
		if ( 1 ){
			chksum = 0;
			for ( i = 0;i < size; i++ )	{
				chksum = chksum + *(unsigned char*)work_address;

				iresult = 1;
				while( iresult == 1 ){
					iresult = Serial_DirectTransmitOneByte( *(unsigned char*)work_address );
				}
				if ( iresult == 3 ) break;
				work_address = (void*)((unsigned int)work_address + 1);
				if ( ( i % 0x1000 ) == 0 ){
					if ( PRGM_GetKey() ) break;
					IntToHex( size - i, hb );
					locate_OS( 3, 4 );
					PrintLine( hb, 19 );
					Bdisp_PutDisp_DD();
				}	
			}
			p = (unsigned char*)&chksum;
			for (j=0;j<2;j++){
				for (i=0;i<4;i++){
					iresult = 1;
					while( iresult == 1 ) iresult = Serial_DirectTransmitOneByte( p[i] );
				}
				if ( iresult ) return;
			}
		}	
		while ( Serial_Close( 0 ) ){
			locate_OS( 3, 5 );
			PrintLine( "wait for trans...", 19 );
			GetKey( &key );
		}	
	}
	MsgBoxPop();
	Cursor_SetFlashOn( 0 );

}


#define MAXEDPOS 15
#define EDSIZE 20 
#define XBOX 3
#define YHEX 3
#define YSTR 5
//
unsigned int SearchMemory( unsigned int&start_address ){
unsigned char work[EDSIZE], binwork[EDSIZE], hb[20], swork[EDSIZE];
int key, i, iContinue=1, idigit, edpos = 0, work_address = start_address;
int maxcomp, iresult = 0;
int xpos=XBOX, ypos=YHEX;
	memset( work, 0, EDSIZE );
	memset( swork, 0, EDSIZE );
	MsgBoxPush( 6 );
	locate_OS( XBOX, YSTR-1 );
	Print( "enter str:" );
	locate_OS( XBOX, YHEX-1 );
	Print( "enter hex:" );
	while (iContinue) {
		locate_OS( XBOX, YSTR );
		PrintLine( swork, MAXEDPOS+3 );
		locate_OS( XBOX, YHEX );
		PrintLine( work, MAXEDPOS+3 );
		locate_OS( XBOX+edpos, ypos );
		GetKey( &key );
		idigit = 0;

		switch (key){
			case KEY_0 : idigit = '0'; break;
			case KEY_1 : idigit = '1'; break;
			case KEY_2 : idigit = '2'; break;
			case KEY_3 : idigit = '3'; break;
			case KEY_4 : idigit = '4'; break;
			case KEY_5 : idigit = '5'; break;
			case KEY_6 : idigit = '6'; break;
			case KEY_7 : idigit = '7'; break;
			case KEY_8 : idigit = '8'; break;
			case KEY_9 : idigit = '9'; break;
			case KEY_A : idigit = 'A'; break;
			case KEY_B : idigit = 'B'; break;
			case KEY_C : idigit = 'C'; break;
			case KEY_D : idigit = 'D'; break;
			case KEY_E : idigit = 'E'; break;
			case KEY_F : idigit = 'F'; break;
			case KEY_RIGHT : if (edpos<MAXEDPOS)edpos++; break;
			case KEY_LEFT : if ( edpos > 0 ) edpos--; break;
			case KEY_UP :
			case KEY_DOWN : if (ypos==YSTR) ypos=YHEX; else ypos=YSTR; edpos = 0; break;
			case KEY_DEL : 
				for (i=edpos;i<20;i++) if (ypos==YSTR) swork[i]=0; else work[i]=0; 
				if ( edpos > 0 ) edpos--; 
				break;
			case KEY_EXE : 
				if (ypos==YHEX){
					maxcomp = edpos / 2;
					for ( i = 0; i < maxcomp; i++){ 
						HexToByte( work+2*i, binwork+i );
					}
				}else{
					maxcomp = edpos;
					memcpy( binwork, swork, maxcomp );
				}
				work_address++;
				locate_OS( XBOX, YSTR+1 );
				PrintLine( "", 19 );
				Bdisp_PutDisp_DD();
				while( !memcmp( (void*)work_address, binwork, maxcomp ) ){
					work_address++;
					if (!work_address) break;
					if ( !(work_address & 0x000FFFFF) ){
						IntToHex( work_address, hb );
						locate_OS( XBOX, YSTR+1 );
						Print( hb );
						Bdisp_PutDisp_DD();
					}
				}
				if ( work_address ){
					start_address = work_address & ~0x03;
					IntToHex( work_address, hb );
					locate_OS( XBOX, YSTR+1 );
					Print( hb );
					iresult = 1;
				}
				break;
			case KEY_EXIT : iContinue = 0; break;
			default: if (ypos==YSTR){
				if (( key >= ' ' ) && ( key <= 'z' )){
					idigit = key;
				}
			};
		}
		if (ypos==YHEX){
			for ( i=0; i < edpos; i++ ) if ( work[ edpos ]==0 ) work[ edpos ]='0';
			if (idigit){
				work[ edpos ] = idigit;
				if ( edpos < MAXEDPOS ) edpos++;
			}
		}else{
			for ( i=0; i < edpos; i++ ) if ( swork[ edpos ]==0 ) swork[ edpos ]=' ';
			if (idigit){
				swork[ edpos ] = idigit;
				if ( edpos < MAXEDPOS ) edpos++;
			}
		}
	}
	MsgBoxPop();

	return iresult;
}

#define TESTADDR 0xA405013C
#define YMAXHELP 6
//
void Help(){
int key, ypos = 2;
	Cursor_SetFlashOff();
	MsgBoxPush( YMAXHELP );
	locate_OS( 3, ypos++ );
	PrintLine( "F1 : help", 19 );
	locate_OS( 3, ypos++ );
	PrintLine( "F2 : search", 19 );
	locate_OS( 3, ypos++ );
	PrintLine( "F3 : backup", 19 );
	locate_OS( 3, ypos++ );
	PrintLine( "F4 : to serial", 19 );
	locate_OS( 3, ypos++ );
	PrintLine( "F6 : hextoggle", 19 );
	locate_OS( 3, ypos++ );
	PrintLine( "EXIT : exit", 19 );
	GetKey( &key );
	MsgBoxPop();
	Cursor_SetFlashOn( 0 );
}

//
unsigned int memviewer( unsigned int*start_address, int mode ){
unsigned char hb[22];
unsigned int iwork;
unsigned short w;
int key, i, j, k, iresult = 0;
unsigned short*p;
unsigned char*pc;
int iContinue = 1;
unsigned char hextoggle = 0;
char editpos = 0;
unsigned int idigit, iRecurrent = 0, ticks;
	
	memset( hb, 0, sizeof( hb ) );
	
	Cursor_SetFlashOn( 0 );
	while (iContinue) {
		if (*start_address < 0x80000000) *start_address=0x80000000;
		hb[8] = 0;
		StatusArea( "INSIGHT memory" );
		
		IntToHex( *start_address, hb );
		locate_OS( 1, 1 );
		PrintLine( hb, 21 );
		
		if (mode==0){
			locate_OS( 10, 1 );
			if (hextoggle) PrintLine( "F6(hex)", 21 );
			else PrintLine( "F6(ascii)", 21 );
			p = (unsigned short*)(*start_address);
			for (i=0;i<7;i++){
				WordToHex( (unsigned int)p & 0xFFFF, hb );
				hb[ 4 ] = ' ';
				if (hextoggle){
					pc = (unsigned char*)p;
					for (j=0;j<8;j++){
						hb[5+j*2] = '.';
						if ((*pc >= 0x20 ) && (*pc < 0x7F ) )
							hb[6+j*2] = *pc;
						else
							hb[6+j*2] = '.';
						pc++;
					}
					p = (unsigned short*)pc;
				}else{
					for (j=0;j<2;j++){
						WordToHex( *p, hb+5+j*4 );
						p++;
					}	
					for (j=0;j<2;j++){
						WordToHex( *p, hb+13+j*4 );
						p++;
					}	
				}
				hb[21] = 0;
				locate_OS( 1, i+2 );
				PrintLine( hb, 21 );
			}

			if ( iRecurrent ){
				key = PRGM_GetKey();
				if ( key != KEY_PRGM_EXIT ){
					continue;
				}
				iRecurrent = 0;
				ticks = RTC_GetTicks();
				while (!RTC_Elapsed_ms( ticks, 100 ));
			}
		}	
		locate_OS( 20, 1 );
		Print( ".." );
		locate_OS( editpos+1, 1 );
		
		GetKey( &key );

		idigit = 0xFFFFFFFF;
		switch ( key ){
		  case KEY_HELP : Help(); break;
		  case KEY_SEARCH : if ( SearchMemory( *start_address) ) editpos = 7; break;
		  case KEY_BACKUP : BackupMemory( *start_address); break;
		  case KEY_SERIAL : MemoryToSerial( *start_address); break;
		  case KEY_HEXTOGGLE : hextoggle ^= 1; break;
		  case KEY_LEFT : if (editpos > 0) editpos--; else editpos=7; break;
		  case KEY_RIGHT : if (editpos < 7) editpos++; else editpos=0; break;
		  case KEY_UP : if (editpos==7) *start_address -= 8; else *start_address -= (1 << (7-editpos)*4); break;
		  case KEY_DOWN : if (editpos==7) *start_address += 8; else *start_address += (1 << (7-editpos)*4); break;
		  case KEY_0 : idigit = 0x0; break;
		  case KEY_1 : idigit = 0x1; break;
		  case KEY_2 : idigit = 0x2; break;
		  case KEY_3 : idigit = 0x3; break;
		  case KEY_4 : idigit = 0x4; break;
		  case KEY_5 : idigit = 0x5; break;
		  case KEY_6 : idigit = 0x6; break;
		  case KEY_7 : idigit = 0x7; break;
		  case KEY_8 : idigit = 0x8; break;
		  case KEY_9 : idigit = 0x9; break;
		  case KEY_A : idigit = 0xA; break;
		  case KEY_B : idigit = 0xB; break;
		  case KEY_C : idigit = 0xC; break;
		  case KEY_D : idigit = 0xD; break;
		  case KEY_E : idigit = 0xE; break;
		  case KEY_F : idigit = 0xF; break;
		  case KEY_RECURRENT : iRecurrent = 1; break;
		  case KEY_EXIT : iContinue = 0; break;
		  case KEY_EXE : iContinue = 0; iresult = 1; break;
		}
		
		if ( idigit != 0xFFFFFFFF ){
			*start_address &= (~(0xF0000000 >> editpos*4));
			*start_address |= (idigit << (7-editpos)*4);
			if (mode==0)
				if ( *start_address & 0x03 ) *start_address = ( *start_address | 0x3 ) + 1;
			if (editpos < 7) editpos++; else editpos=0;
		}	
	}
	Cursor_SetFlashOff();
	return iresult;
}

}

